/*
Copyright (C) 2007 - Nastasi Christian & Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "mySerial.h"
#include <iostream>
#define TIMEOUT_INT 		100 //Byte interarraival timeout (ms)
#define TIMEOUT_ELEMENT 	250	//Time distance between 2 adjacent time divisor (ms)
#define TIMEOUT_START		100	//Start (timeouts value with divisor = 0) (ms)

using namespace std;

int my_Serial_init(My_Serial* com, const char* port_name, unsigned int baud_rate)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	com->handlerCom = CreateFile(port_name, 
		GENERIC_READ | GENERIC_WRITE,
		3, // exclusive access
		NULL, // no security
		OPEN_EXISTING,
		0, // no overlapped I/O -> blocking i/o
		NULL
	); // null template 

	if (com->handlerCom == INVALID_HANDLE_VALUE)
		return 0;
 
	com->bPortReady = SetupComm(com->handlerCom, 128, 128); // set buffer sizes
 
 
	com->bPortReady = GetCommState(com->handlerCom, &(com->ctrl));
	(com->ctrl).BaudRate = baud_rate;
	(com->ctrl).ByteSize = 8;
	(com->ctrl).Parity = NOPARITY;
	// (com->ctrl).Parity = EVENPARITY;
	(com->ctrl).StopBits = ONESTOPBIT;
	(com->ctrl).fAbortOnError = TRUE;
 
	// set XON/XOFF
	(com->ctrl).fOutX = FALSE; // XON/XOFF off for transmit
	(com->ctrl).fInX = FALSE; // XON/XOFF off for receive
	// set RTSCTS
	(com->ctrl).fOutxCtsFlow = TRUE; // turn on CTS flow control
	(com->ctrl).fRtsControl = RTS_CONTROL_HANDSHAKE; // 
	// set DSRDTR
	(com->ctrl).fOutxDsrFlow = FALSE; // turn on DSR flow control
	(com->ctrl).fDtrControl = DTR_CONTROL_ENABLE; // 
	// (com->ctrl).fDtrControl = DTR_CONTROL_DISABLE; // 
	// (com->ctrl).fDtrControl = DTR_CONTROL_HANDSHAKE; // 
 
	com->bPortReady = SetCommState(com->handlerCom, &(com->ctrl));
 
	// Communication timeouts are optional
	com->bPortReady = GetCommTimeouts (com->handlerCom, &(com->CommTimeouts));
	
	//The maximum time allowed to elapse between the arrival of two bytes on the communications line, in milliseconds.
	// The time period begins when the first byte is received.
	(com->CommTimeouts).ReadIntervalTimeout = TIMEOUT_INT;
	
	//Set the maximum timeouts
	my_Serial_set_timeout(com, 7);
	
	(com->CommTimeouts).WriteTotalTimeoutConstant = 10000;
	(com->CommTimeouts).WriteTotalTimeoutMultiplier = 10000;
 
	com->bPortReady = SetCommTimeouts (com->handlerCom, &(com->CommTimeouts));

	if(com->bPortReady == FALSE)
		return 0;

	return 1;
}
#else /* UNIX Implementation */
{
	struct termios options;
	speed_t baud;

	com->fd = open(port_name, 
		O_RDWR 		// Read-Write I/O
		| O_NOCTTY 	// Refers to device terminal
		| O_SYNC	// Blocking I/O 
	); 

	if (com->fd == -1) 
		return 0;
	else
		fcntl(com->fd, F_SETFL, 0);

	// Get the current options for the port...
	tcgetattr(com->fd, &options);
	// Set the baud rates to...
	switch(baud_rate)
	{
		case 9600:
			baud=B9600;
			break;
		/*TODO: use other values!!!*/
		default:
			return 0;
	}

	cfsetispeed(&options, baud);
	cfsetospeed(&options, baud);

	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(com->fd, TCSANOW, &options);

	return 1;
}
#endif

/*
Inizializzazione della seriale personalizzata al divisore di tempo della camera HV7131GP
NB Solo implementazione per l'Api di Windows
*/
int my_Serial_init_HV7131GP(My_Serial* com, const char* port_name, unsigned int baud_rate, int div)
{
	com->handlerCom = CreateFile(port_name, 
		GENERIC_READ | GENERIC_WRITE,
		3, // exclusive access
		NULL, // no security
		OPEN_EXISTING,
		0, // no overlapped I/O -> blocking i/o
		NULL
	); // null template 

	if (com->handlerCom == INVALID_HANDLE_VALUE)
		return 0;
 
	com->bPortReady = SetupComm(com->handlerCom, 128, 128); // set buffer sizes
 
 
	com->bPortReady = GetCommState(com->handlerCom, &(com->ctrl));
	(com->ctrl).BaudRate = baud_rate;
	(com->ctrl).ByteSize = 8;
	(com->ctrl).Parity = NOPARITY;
	// (com->ctrl).Parity = EVENPARITY;
	(com->ctrl).StopBits = ONESTOPBIT;
	(com->ctrl).fAbortOnError = TRUE;
 
	// set XON/XOFF
	(com->ctrl).fOutX = FALSE; // XON/XOFF off for transmit
	(com->ctrl).fInX = FALSE; // XON/XOFF off for receive
	// set RTSCTS
	(com->ctrl).fOutxCtsFlow = TRUE; // turn on CTS flow control
	(com->ctrl).fRtsControl = RTS_CONTROL_HANDSHAKE; // 
	// set DSRDTR
	(com->ctrl).fOutxDsrFlow = FALSE; // turn on DSR flow control
	(com->ctrl).fDtrControl = DTR_CONTROL_ENABLE; // 
	// (com->ctrl).fDtrControl = DTR_CONTROL_DISABLE; // 
	// (com->ctrl).fDtrControl = DTR_CONTROL_HANDSHAKE; // 
 
	com->bPortReady = SetCommState(com->handlerCom, &(com->ctrl));
 
	// Communication timeouts are optional
	com->bPortReady = GetCommTimeouts (com->handlerCom, &(com->CommTimeouts));
	
	//The maximum time allowed to elapse between the arrival of two bytes on the communications line, in milliseconds.
	// The time period begins when the first byte is received.
	(com->CommTimeouts).ReadIntervalTimeout = TIMEOUT_INT;
	
	//ReadTotalTimeoutConstant and ReadTotalTimeoutMultiplier init
	my_Serial_set_timeout(com, div);
	
	(com->CommTimeouts).WriteTotalTimeoutConstant = 10000;
	(com->CommTimeouts).WriteTotalTimeoutMultiplier = 10000;
 
	com->bPortReady = SetCommTimeouts (com->handlerCom, &(com->CommTimeouts));

	if(com->bPortReady == FALSE)
		return 0;

	return 1;
}


void my_Serial_close(My_Serial* com)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	CloseHandle(com->handlerCom);
}
#else /* UNIX Implementation */
{
	close(com->fd);
}
#endif


int my_Serial_getc(My_Serial* com, char *data)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	DWORD read_size;
	if( ReadFile(com->handlerCom, data, sizeof(char), &read_size, NULL) == FALSE )
		return -1;
	else
		return read_size;
}
#else /* UNIX Implementation */
{
	int n = read(com->fd, data, sizeof(char));
	/*if(n<0) n=0;*/
	return n;
}
#endif


int my_Serial_putc(My_Serial* com, char data)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	DWORD write_size;
	if(! WriteFile(com->handlerCom, &data, sizeof(char), &write_size, NULL) )
		return -1;
	else
		return write_size;
}
#else /* UNIX Implementation */
{
	int n = write(com->fd, &data, sizeof(char));
	/*if(n<0) n=0;*/
	return n;
}
#endif

int my_Serial_read(My_Serial* com, char *data, unsigned int data_size)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	DWORD read_size=0;
	if( ReadFile(com->handlerCom, data, sizeof(char)*data_size, &read_size, NULL) == FALSE )
		return -1;
	else
		return read_size;
}
#else /* UNIX Implementation */
{
	int n = read(com->fd, data, sizeof(char)*data_size);
	/*if(n<0) n=0;*/
	return n;
}
#endif

/*
  Invio di una stringa di caratteri sul bus seriale...
*/
int my_Serial_write(My_Serial* com, char* data, unsigned int data_size)
#ifdef __CYGWIN__ /* WIN32 Implementation */
{
	DWORD write_size;
	if( !WriteFile(com->handlerCom, data, sizeof(char)*data_size, &write_size, NULL) )	
		return -1;
	else
		return write_size;
}
#else /* UNIX Implementation */
{
	int n = write(com->fd, data, sizeof(char)*data_size);
	/*if(n<0) n=0;*/
	return n;
}
#endif
/*
Fissa i timeout della seriale in base ai divisori di tempo della camera HV7131GP
NB Solo implementazione per l'Api di Windows
*/
void my_Serial_set_timeout(My_Serial* com, int div)
{
	//A constant used to calculate the total time-out period for read operations, in milliseconds. 
	//For each read operation, this value is added to the product of the ReadTotalTimeoutMultiplier member and the requested number of bytes.
	(com->CommTimeouts).ReadTotalTimeoutConstant = div * TIMEOUT_ELEMENT + TIMEOUT_START;
	
	//The multiplier used to calculate the total time-out period for read operations, in milliseconds. 
	//For each read operation, this value is multiplied by the requested number of bytes to be read.
	(com->CommTimeouts).ReadTotalTimeoutMultiplier = div * TIMEOUT_ELEMENT + TIMEOUT_START;
	
}
